@@ -4,7 +4,6 @@ import sys
 from contextlib import contextmanager
 
 import py.code
-
 import pytest
 
 pytest_plugins = 'pytester'
@@ -14,6 +13,19 @@ skip_pypy = pytest.mark.skipif(platform.python_implementation() == 'PyPy',
                                reason='could not make work on pypy')
 
 
+@pytest.fixture
+def needs_assert_rewrite(pytestconfig):
+    """
+    Fixture which skips requesting test if assertion rewrite is disabled (#102)
+
+    Making this a fixture to avoid acessing pytest's config in the global context.
+    """
+    option = pytestconfig.getoption('assertmode')
+    if option != 'rewrite':
+        pytest.skip('this test needs assertion rewrite to work but current option '
+                    'is "{}"'.format(option))
+
+
 class UnixFS(object):
     """
     Wrapper to os functions to simulate a Unix file system, used for testing
@@ -69,10 +81,8 @@ def mock_using_patch(mocker):
 
 
 def mock_using_patch_multiple(mocker):
-    from pytest_mock import mock_module
-
-    r = mocker.patch.multiple('os', remove=mock_module.DEFAULT,
-                              listdir=mock_module.DEFAULT)
+    r = mocker.patch.multiple('os', remove=mocker.DEFAULT,
+                              listdir=mocker.DEFAULT)
     return r['remove'], r['listdir']
 
 
@@ -89,6 +99,8 @@ def test_mock_patches(mock_fs, mocker, check_unix_fs_mocked):
     mock_fs(mocker)
     mocked_rm, mocked_ls = mock_fs(mocker)
     check_unix_fs_mocked(mocked_rm, mocked_ls)
+    mocker.resetall()
+    mocker.stopall()
 
 
 def test_mock_patch_dict(mocker):
@@ -103,23 +115,16 @@ def test_mock_patch_dict(mocker):
     assert x == {'original': 1}
 
 
-def test_mock_fixture_is_deprecated(testdir):
+def test_mock_patch_dict_resetall(mocker):
     """
-    Test that a warning emitted when using deprecated "mock" fixture.
+    We can call resetall after patching a dict.
+    :param mock:
     """
-    testdir.makepyfile('''
-        import warnings
-        import os
-        warnings.simplefilter('always')
-
-        def test_foo(mock, tmpdir):
-            mock.patch('os.listdir', return_value=['mocked'])
-            assert os.listdir(str(tmpdir)) == ['mocked']
-            mock.stopall()
-            assert os.listdir(str(tmpdir)) == []
-    ''')
-    result = testdir.runpytest('-s')
-    result.stderr.fnmatch_lines(['*"mock" fixture has been deprecated*'])
+    x = {'original': 1}
+    mocker.patch.dict(x, values=[('new', 10)], clear=True)
+    assert x == {'new': 10}
+    mocker.resetall()
+    assert x == {'new': 10}
 
 
 def test_deprecated_mock(mock, tmpdir):
@@ -133,10 +138,12 @@ def test_deprecated_mock(mock, tmpdir):
 
 
 @pytest.mark.parametrize('name', ['MagicMock', 'PropertyMock', 'Mock', 'call', 'ANY', 'sentinel', 'mock_open'])
-def test_mocker_aliases(name):
-    from pytest_mock import mock_module, MockFixture
+def test_mocker_aliases(name, pytestconfig):
+    from pytest_mock import _get_mock_module, MockFixture
 
-    mocker = MockFixture()
+    mock_module = _get_mock_module(pytestconfig)
+
+    mocker = MockFixture(pytestconfig)
     assert getattr(mocker, name) is getattr(mock_module, name)
 
 
@@ -203,8 +210,6 @@ def test_instance_method_spy(mocker):
 
 @skip_pypy
 def test_instance_method_by_class_spy(mocker):
-    from pytest_mock import mock_module
-
     class Foo(object):
 
         def bar(self, arg):
@@ -215,13 +220,12 @@ def test_instance_method_by_class_spy(mocker):
     other = Foo()
     assert foo.bar(arg=10) == 20
     assert other.bar(arg=10) == 20
-    calls = [mock_module.call(foo, arg=10), mock_module.call(other, arg=10)]
+    calls = [mocker.call(foo, arg=10), mocker.call(other, arg=10)]
     assert spy.call_args_list == calls
 
 
 @skip_pypy
 def test_instance_method_by_subclass_spy(mocker):
-    from pytest_mock import mock_module
 
     class Base(object):
 
@@ -236,7 +240,7 @@ def test_instance_method_by_subclass_spy(mocker):
     other = Foo()
     assert foo.bar(arg=10) == 20
     assert other.bar(arg=10) == 20
-    calls = [mock_module.call(foo, arg=10), mock_module.call(other, arg=10)]
+    calls = [mocker.call(foo, arg=10), mocker.call(other, arg=10)]
     assert spy.call_args_list == calls
 
 
@@ -357,8 +361,8 @@ def assert_argument_introspection(left, right):
         raise AssertionError("DID NOT RAISE")
 
 
-@pytest.mark.skipif(sys.version_info[:2] in [(3, 3), (3, 4)],
-                    reason="assert_not_called not available in python 3.3 and 3.4")
+@pytest.mark.skipif(sys.version_info[:2] == (3, 4),
+                    reason="assert_not_called not available in Python 3.4")
 def test_assert_not_called_wrapper(mocker):
     stub = mocker.stub()
     stub.assert_not_called()
@@ -384,6 +388,7 @@ def test_assert_called_once_with_wrapper(mocker):
         stub.assert_called_once_with("foo")
 
 
+@pytest.mark.usefixtures('needs_assert_rewrite')
 def test_assert_called_args_with_introspection(mocker):
     stub = mocker.stub()
 
@@ -399,6 +404,7 @@ def test_assert_called_args_with_introspection(mocker):
         stub.assert_called_once_with(*wrong_args)
 
 
+@pytest.mark.usefixtures('needs_assert_rewrite')
 def test_assert_called_kwargs_with_introspection(mocker):
     stub = mocker.stub()
 
@@ -424,12 +430,11 @@ def test_assert_any_call_wrapper(mocker):
 
 
 def test_assert_has_calls(mocker):
-    from pytest_mock import mock_module
     stub = mocker.stub()
     stub("foo")
-    stub.assert_has_calls([mock_module.call("foo")])
+    stub.assert_has_calls([mocker.call("foo")])
     with assert_traceback():
-        stub.assert_has_calls([mock_module.call("bar")])
+        stub.assert_has_calls([mocker.call("bar")])
 
 
 def test_monkeypatch_ini(mocker, testdir):
@@ -447,11 +452,7 @@ def test_monkeypatch_ini(mocker, testdir):
         [pytest]
         mock_traceback_monkeypatch = false
     """)
-    if hasattr(testdir, 'runpytest_subprocess'):
-        result = testdir.runpytest_subprocess()
-    else:
-        # pytest 2.7.X
-        result = testdir.runpytest()
+    result = runpytest_subprocess(testdir)
     assert result.ret == 0
 
 
@@ -487,14 +488,75 @@ def test_monkeypatch_native(testdir):
             stub(1, greet='hello')
             stub.assert_called_once_with(1, greet='hey')
     """)
-    if hasattr(testdir, 'runpytest_subprocess'):
-        result = testdir.runpytest_subprocess('--tb=native')
-    else:
-        # pytest 2.7.X
-        result = testdir.runpytest('--tb=native')
+    result = runpytest_subprocess(testdir, '--tb=native')
     assert result.ret == 1
     assert 'During handling of the above exception' not in result.stdout.str()
     assert 'Differing items:' not in result.stdout.str()
     traceback_lines = [x for x in result.stdout.str().splitlines()
                        if 'Traceback (most recent call last)' in x]
     assert len(traceback_lines) == 1  # make sure there are no duplicated tracebacks (#44)
+
+
+@pytest.mark.skipif(sys.version_info[0] < 3, reason='Py3 only')
+def test_standalone_mock(testdir):
+    """Check that the "mock_use_standalone" is being used.
+    """
+    testdir.makepyfile("""
+        def test_foo(mocker):
+            pass
+    """)
+    testdir.makeini("""
+        [pytest]
+        mock_use_standalone_module = true
+    """)
+    result = runpytest_subprocess(testdir)
+    assert result.ret == 3
+    result.stderr.fnmatch_lines([
+        "*No module named 'mock'*",
+    ])
+
+
+def runpytest_subprocess(testdir, *args):
+    """Testdir.runpytest_subprocess only available in pytest-2.8+"""
+    if hasattr(testdir, 'runpytest_subprocess'):
+        return testdir.runpytest_subprocess(*args)
+    else:
+        # pytest 2.7.X
+        return testdir.runpytest(*args)
+
+
+@pytest.mark.usefixtures('needs_assert_rewrite')
+def test_detailed_introspection(testdir):
+    """Check that the "mock_use_standalone" is being used.
+    """
+    testdir.makepyfile("""
+        def test(mocker):
+            m = mocker.Mock()
+            m('fo')
+            m.assert_called_once_with('', bar=4)
+    """)
+    result = testdir.runpytest('-s')
+    result.stdout.fnmatch_lines([
+        "*AssertionError: Expected call: mock('', bar=4)*",
+        "*Actual call: mock('fo')*",
+        "*pytest introspection follows:*",
+        '*Args:',
+        "*assert ('fo',) == ('',)",
+        "*At index 0 diff: 'fo' != ''*",
+        "*Use -v to get the full diff*",
+        "*Kwargs:*",
+        "*assert {} == {'bar': 4}*",
+        "*Right contains more items:*",
+        "*{'bar': 4}*",
+        "*Use -v to get the full diff*",
+    ])
+
+
+def test_assert_called_with_unicode_arguments(mocker):
+    """Test bug in assert_call_with called with non-ascii unicode string (#91)"""
+    stub = mocker.stub()
+    stub(b'l\xc3\xb6k'.decode('UTF-8'))
+
+    with pytest.raises(AssertionError):
+        stub.assert_called_with(u'lak')
+
